#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "filter.h"
#include "die.h"
#include "strtools.h"

/*
 * - Strip off comments. A comment starts with a # character and
 *   extends to the end of the line.
 *
 * - Print the second word (the IP address) if present, one space,
 *   the first word (allow, deny or similar) if present, then the rest
 *   of the line:  "keyword addr rest" -> "addr keyword rest"
 */
static char *aclpp_rewrite( char *input )
{
	static char *buffer = NULL;

	char *p;
	size_t input_len = strlen( input );
	char *input_end;
	char *key_start, *key_end, *addr_start, *addr_end;

	if ( buffer != NULL )
		free( buffer );
	buffer = malloc_or_die( input_len + 1 );

	input_end = strchr( input, '#' );
	if ( input_end == NULL )
		input_end = input + input_len;

	key_start = skip_whitespace( input, input_end );
	key_end = skip_word( key_start, input_end );

	addr_start = skip_whitespace( key_end, input_end );
	addr_end = skip_word( addr_start, input_end );

	p = buffer;

	while ( addr_start < addr_end )
		*p++ = *addr_start++;

	*p++ = ' ';

	while ( key_start < key_end )
		*p++ = *key_start++;

	while ( addr_end < input_end )
		*p++ = *addr_end++;

	*p++ = '\0';

	return buffer;
}


int main( int argc, char *argv[] )
{
	char *progname = *argv;

	/* Option defaults:
	 */
	int add_dot = 0;

	++argv;
	--argc;

	while ( argc && **argv == '-' ) {

		if ( strcmp( *argv, "-dot" ) == 0 )
			++add_dot;
		else
			die( "\nUsage: %s [ -dot ] [ file ... ]\n", progname );

		++argv;
		--argc;
	}

	if ( argc ) {

		while ( argc ) {

			FILE *f = fopen( *argv, "r" );

			if ( f == NULL )
				die_io( *argv );

			read_input( f, *argv, aclpp_rewrite );

			fclose( f );

			++argv;
			--argc;
		}
	}
	else {
		read_input( stdin, "stdin", aclpp_rewrite );
	}

	write_output( stdout, "stdout", add_dot, NULL );

	return 0;
}
