#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "getline.h"


#define MIN(A, B)       ((A) < (B) ? (A) : (B))
#define MAX(A, B)       ((A) < (B) ? (B) : (A))


static void *resize_array( void *start, int elem_size, int oldlen, int newlen )
{
	void *p = NULL;

	if ( newlen != 0 )
		if ((p = malloc(newlen * elem_size)) == NULL)  return NULL;

	if (start != NULL) {

		if (newlen != 0) {
			memcpy( p, start, MIN(oldlen, newlen) * elem_size );
		}

		free(start);
	}

	return p;
}


/* Read an arbitrary long line of text from a FILE by reading characters
 * until a \n is seen. Return a pointer to a null-terminated buffer
 * containing the text without the \n.
 *
 * If EOF is encountered, any text read before the EOF is returned as usual,
 * but if no characters were read before the EOF, then NULL is returned.
 *
 * If en error is encountered, NULL is returned. Programs may call feof()
 * after NULL is returned to see if there is an error or EOF condition.
 *
 */
char *getline( FILE *f )
{
	static size_t buflen = 0;
	static char *buffer = NULL;

	int n = 0;    /* The number of characters read so far */
	int c;

	while ( 1 ) {

		if ( n == buflen ) {

			size_t newlen = (128 + buflen) * 2;

			buffer = resize_array( buffer, 1, buflen, newlen );

			if ( buffer == NULL )
				return NULL;

			buflen = newlen;
		}

		c = getc( f );

		if ( c == '\n' )
			break;

		if ( c == EOF ) {

			if ( n == 0 || ferror( f ) )
				return NULL;

			break;
		}

		buffer[ n++ ] = c;
	}

	buffer[n] = '\0';

	return buffer;
}
